import java.util.*;

// PointInfo class:
// For saving position and ball type.
class PointInfo {
  public int row;
  public int col;
  public int ballType;

  public PointInfo(int r, int c, int t) { row = r; col = c; ballType = t; }
}

// Board class:
// This knows all about the rule!
public class Board {
  private int iRows, iCols;
  private int [][] iBoard;
  private int [][] iFlags;
  private int iLeftBalls;
  private int iPicked = 0;
  private int iBallTypes;

  public Board(int rows, int cols, int ballTypes)
  {
    iRows = rows;
    iCols = cols;
    iLeftBalls = iRows * iCols;
    iBoard = new int[iRows][iCols];
    iFlags = new int[iRows][iCols];

    java.util.Random random = new java.util.Random();

    for (int row = 0; row < iRows; ++row) 
      for (int col = 0; col < iCols; ++col)
	iBoard[row][col] = random.nextInt(ballTypes) + 1;
    iBallTypes = ballTypes;
  }

  public int rows() { return iRows; }
  public int columns() { return iCols; }
  public int types() { return iBallTypes; }

  // Returns the contents of the cell.
  // 0 if there is no ball or out of range,
  // otherwise a number in the range { 1, 2, ... NUM_COLORS }
  public int get(int row, int col) 
  {
    if (row < 0 || row > iRows - 1 || col < 0 || col > iCols - 1) return 0;
    return iBoard[row][col];
  }

  /* The user has clicked on a ball.
     This method should update the board, and return the
     number of balls deleted. */
  public int pick(int row, int col)
  {
    if (iBoard[row][col] == 0)
      return 0;  // nothing there

    // implementation of of our project
    iPicked = iBoard[row][col];
    int n = clearBall(row, col, -1);
    // gravity effect
    updateBoard();
    // check bonus score / calculate score
    return ((iLeftBalls == 0) ? 500 : 0) + n*(n+1)/2;
  }
  
  /* Determine the end of game */
  public boolean isFinished()
  {
    // there's no balls,
    if (iLeftBalls == 0) return true;
    // if at least one ball has its same neighbor, game is not finished.
    int i, j;
    for (i = 0; i < iRows; ++i)
      for (j = 0; j < iCols; ++j) {
	if (i < iRows - 1)
	  if (iBoard[i][j] != 0 && iBoard[i][j] == iBoard[i+1][j]) return false;
	if (j < iCols - 1)
	  if (iBoard[i][j] != 0 && iBoard[i][j] == iBoard[i][j+1]) return false;
      }
    return true;
  }

  // return the group of same balls same to given point's
  public List<PointInfo> getGroup(int r, int c)
  {
    List<PointInfo> s = new ArrayList<PointInfo>();
    if (0 <= r && r <= iRows-1 && 0 <= c && c <= iCols-1 && !(iBoard[r][c] == 0)) {
      iFlags = new int[iRows][iCols];
      recurseGroup(r, c, iBoard[r][c], s);
      if (s.size() == 1) s.clear(); // if only 1 ball, do not make it group
    }
    return s;
  }
  
  // ------------------------------------------------------------------------
  
  // remove a ball (row, col)
  private void removeRowOfCol(int r, int col)
  {
    int i;
    for (i=r; i<iRows -1; i++) {
      iBoard[i][col] = iBoard[i+1][col];
      iBoard[i+1][col] = 0;
    }
  }

  // remove a column
  private void removeColumn(int c)
  {
    int r, i;
    for (i = c; i < iCols - 1; i++) {
      for (r = 0; r < iRows; r++) {
        iBoard[r][i] = iBoard[r][i + 1];
        iBoard[r][i + 1] = 0;
      }
    }
  }

  // get the number of balls in a column
  private int getNumOfBalls(int col)
  {
    int i, n = 0;
    for(i=0; i<iRows; i++)
      if (iBoard[i][col] != 0) ++n;
    return n;
  }

  // compact the content of the board by the rule
  private void updateBoard()
  {
    // make gravity effect
    int r, r2, c, c2;
    for(c = 0; c < iCols; c++)
      for(r = 0; r < iRows - 1; r++)
	for(r2 = r + 1; r2 < iRows; r2++)
	  if (iBoard[r][c] == 0) removeRowOfCol(r, c);

    // remove empty columns
    for(c = 0; c < iCols - 1; c++)
      for(c2 = c + 1; c2 < iCols; c2++)
	if (getNumOfBalls(c) == 0)
	  removeColumn(c);
  }

  // clear balls! (Recursion)
  private int clearBall(int row, int col, int isSingle)
  {

    int count = 0, b = isSingle;

    // check if the ball is isolated
    if (0 < row)
      if (iBoard[row - 1][col] == iPicked) b += 1;
    if (row < iRows - 1)
      if (iBoard[row + 1][col] == iPicked) b += 1;
    if (0 < col)
      if (iBoard[row][col - 1] == iPicked) b += 1;
    if (col < iCols - 1)
      if (iBoard[row][col + 1] == iPicked) b += 1;
    if(b < 0) return 0;

    // remove the ball
    if (0 <= row && row < iRows && 0 <= col && col < iCols && iBoard[row][col] == iPicked) {
      iBoard[row][col] = 0;
      --iLeftBalls;
      count = 1;
    }

    // recurse 4 balls around this
    if (0 < row)
      if (iBoard[row - 1][col] == iPicked) count += clearBall(row - 1, col, isSingle + 1);
    if (row < iRows - 1)
      if (iBoard[row + 1][col] == iPicked) count += clearBall(row + 1, col, isSingle + 1);
    if (0 < col)
      if (iBoard[row][col - 1] == iPicked) count += clearBall(row, col - 1, isSingle + 1);
    if (col < iCols - 1)
      if (iBoard[row][col + 1] == iPicked) count += clearBall(row, col + 1, isSingle + 1);

    return count;
  }

  // another recursion: only get the group of balls (no removes)
  private void recurseGroup(int row, int col, int picked, List<PointInfo> s)
  {

    if (iFlags[row][col] == 1) return;
    
    // flags the ball
    if (0 <= row && row < iRows && 0 <= col && col < iCols && iBoard[row][col] == picked) {
      iFlags[row][col] = 1;
      s.add(new PointInfo(row, col, picked));
    }

    // recurse 4 balls around this
    if (0 < row)
      if (iBoard[row - 1][col] == picked) recurseGroup(row - 1, col, picked, s);
    if (row < iRows - 1)
      if (iBoard[row + 1][col] == picked) recurseGroup(row + 1, col, picked, s);
    if (0 < col)
      if (iBoard[row][col - 1] == picked) recurseGroup(row, col - 1, picked, s);
    if (col < iCols - 1)
      if (iBoard[row][col + 1] == picked) recurseGroup(row, col + 1, picked, s);
  }

}

/* vim: set ts=8 sw=2 sts=2 noet: */
